      subroutine adja(delta)
c
c    adjusts computer clock by number of seconds
c    specified in input parameter delta.  this is
c    the version for the openVMS on the alpha/axp.
c
c     the clock is read and the low portion of the time
c     is incremented or decremented by 100 ms (100 ms =
c     1000000 least counts).  the result is then written
c     back into the system time.  since the time required
c     to update the system clock is on the order of 10 ms,
c     the program waits until the time has just changed to
c     maximize the interval before the next tick
c
c     the time is also not changed if the increment or 
c     decrement would result in an overflow or underflow
c     of the least significant longword of the time quad
c     word
c
c
      integer*4 itime(2)
c
c     following location holds low portion of time just
c     read
c
      integer*4 lstime
c
c    since the low portion of the time is an unsigned 32-bit
c    quantity, its overflow cannot be checked by arithmetic
c    since fortran does not have an unsigned 32 bit integer.
c
c    When the clock is advanced each loop adds 1000000, 
c    which is 000f4240 in hex, so that there will not be 
c    an overflow if the least significant portion of the time 
c    is fff00000 or less, where this quantity must be thought of 
c    as unsigned. This number is negative from the signed
c    point of view.  imax below has a safety factor.
c
      integer*4 imax
      data imax /'ffc00000'x/
      real*8 delta
c
      include '($syssrvnam)'
c
c
c    get rounded number of adjustments at 100 msec. each
c    where input time parameter delta is in seconds
c
      ntick=10.*(abs(delta) + 0.05)
      if(delta .gt. 0) then
        idt=1000000             !increment the time
      else
        idt=-1000000            !decrement the time
      endif
c
c     get low portion of time now so that loops
c     can wait for it to change
c
      istat=sys$gettim(%ref(itime))
      if(istat .ne. 1) call lib$stop(%val(istat))
      lstime=itime(1)
      do 16 i=1,ntick
    6 istat=sys$gettim(%ref(itime))
      if(istat .ne. 1) call lib$stop(%val(istat))
      if(itime(1) .eq. lstime) go to 6
c
c     overflow on increment is when itime is negative and 
c     is between imax and 0
c     underflow on decrement is when itime is positive and
c     is between 1000000 and 0
c
      if( (idt .gt. 0) .and.
     +    (itime(1) .lt. 0) .and.
     +    (itime(1) .gt. imax) ) go to 6
      if( (idt .lt. 0) .and.
     +    (itime(1) .gt. 0) .and.
     +    (itime(1) .lt. 1000000) ) go to 6
      itime(1)=itime(1) + idt
      istat=sys$setime(%ref(itime))
      if(istat .ne. 1) call lib$stop(%val(istat))
c
c    update lstime
c
      istat=sys$gettim(%ref(itime))
      if(istat .ne. 1) call lib$stop(%val(istat))
      lstime=itime(1)
   16 continue
      return
      end
