      program nistim
c
c     this program compares the local time with
c     the time received by calling the nist acts
c     service.  it is much simpler than the ibm pc
c     version since it uses the local machine VMS system
c     for many of the usual system services
c
c     VAX/VMS VERSION 5  23 November 1992
c
c     This version has improved handling of transmission errors,
c     especially for the DST parameter.
c     it also corrects a problem with dst conversion near
c     the transition time
c
c     ibb is used for reading/setting the port
c     characteristics. force it to be word-aligned
c
c
      byte ibb(8)
      integer*4 idum(2)
      equivalence(ibb(1),idum(1))
c
c     following variables hold telephone number and
c     requested operations as read from config. file
c
      byte telno(40),param(20)
c
      byte ians
      character*20 iport
c
c     following constants control what happens
c     each operation is enabled if constant is 1, disabled if 0
c     note that any combination of operations may be specified
c
c
c     ilist     list lines received from nist
c     ichk      compare nist time with system time and type difference
c     iarc      compare nist time with system time and write difference
c               to file nistime.dif  in append mode
c     iset      set system time to nist time
c     idebug    turn on diagnostic messages for debugging
c     irate     compare current difference with previous value to get rate
c
      common/ops/ilist,ichk,iarc,iset,idebug,irate
      common/local/iutdif,idst
c
c     include file holds definition of channel number and various
c     i/o operation codes
c
      include 'iochan2.inc'
c
c     start out with all operations disabled
c
      ilist=0
      ichk=0
      iarc=0
      iset=0
      idebug=0
      irate=0
c
c    open configuration file and read parameters
c
c    first line of configuration file has full telephone number
c    including leading t/p for tone or pulse dialing and any
c    required long distance and accounting information
c
      open(unit=2,name='nistime.cfg',type='old',
     + access='sequential',form='formatted',
     + carriagecontrol='list',readonly,dispose='save')
      read(2,1) il,(telno(i),i=1,il)
      telno(il+1)=0
    1 format(q,40a1)
c
c     second line of configuration file has port number in first
c     two places followed by as many operation code letters as
c     needed
c
c    port number may be specifed as <letter><number>
c    to specify port tt<letter><number>: or as --
c    to indicate that the full name of the port is
c    specified on the last line of the file.  this
c    latter capability will be needed for a port that
c    is not a tt but a tx or whatever.
c
      read(2,1) il,(param(i),i=1,il)
c
c     first two characters are port specifier or
c     -- to show that port will be specified later
c
      param(1)= param(1) .and. '177'o
      param(2)= param(2) .and. '177'o
      if( (param(1) .eq. '-') .and.
     +    (param(2) .eq. '-') ) then
        iport=' '
      else
        encode(5,4,iport) param(1),param(2)
    4   format('TT',2a1,':')
        ilp=5
      endif
c
c     now set operation using specified parameters
c
c     turn off ms bit of each character and convert
c     to upper case for comparison
c
      if(il .ge. 3) then
      do 3 i=3,il
      param(i)= param(i) .and. '177'o
      if(param(i) .gt. '140'o) param(i)=param(i) - '40'o
      if(param(i) .eq. 'L') ilist=1
      if(param(i) .eq. 'A') iarc=1
      if(param(i) .eq. 'C') ichk=1
      if(param(i) .eq. 'S') iset=1
      if(param(i) .eq. 'D') idebug=1
      if(param(i) .eq. 'R') irate=1
    3 continue
      endif
c
c     third line of configuration file has time zone as 
c     letter p/P pACIFIC
c            m/M mOUNTAIN
c            c/C cENTRAL
c            e/E eASTERN
c            z/Z gREENWICH 
c     OR AS (SIGNED) NUMBER: +/- J, GIVING DIFFERENCE BETWEEN
c     LOCAL TIME AND gREENWICH.  lOCATIONS wEST OF gREENWICH ARE
c     NEGATIVE, eAST ARE pOSITIVE.
c
c     OPTIONAL TRAILING d/D, SPECIFIES CONVERSION TO dAYLIGHT 
c     SAVINGS TIME IF PRESENT.
c
      read(2,1) il,(param(i),i=1,il)
      do 5 i=1,il
      param(i)= param(i) .and. '177'o
      if(param(i) .gt. '140'o) param(i)=param(i) - '40'o
    5 continue
      do 9 i=il+1,10
      param(i) = 0
    9 continue
c
c    if first character is a zone letter then decode it
c    and check if second character is daylight savings flag
c   
c    if first character is a sign or digit, then parse number
c
      if(param(1) .eq. 'P') then
        iutdif=-8
      elseif(param(1) .eq. 'M') then
        iutdif=-7
      elseif(param(1) .eq. 'C') then
        iutdif=-6
      elseif(param(1) .eq. 'E') then
        iutdif=-5
      elseif(param(1) .eq. 'Z') then
        iutdif=0
      elseif(param(1) .eq. '-') then
        isign=-1
        iutdif=0
        go to 6
      elseif(param(1) .eq. '+') then
        isign=+1
        iutdif=0
        go to 6
      elseif((param(1).ge. '0').and. (param(1).le. '9'))then
        isign=+1
        iutdif=(param(1) - '0')
        go to 6
      else
        type 7
    7 format(' Error on third line of configuration file.')
      stop
      endif
c
c    if we come here then we decoded time zone as a single letter
c    now look at next character to see about daylight savings time
c
      is=2
      go to 11
c
c    come here if first character was sign or digit -- parse
c    following characters as utc offset as number
c
    6 is=2
      if( (param(2) .ge. '0') .and. (param(2) .le. '9'))then
        iutdif= 10*iutdif + (param(2) - '0')
        is=3
      endif
      if( (param(3) .ge. '0') .and. (param(3) .le. '9'))then
        iutdif= 10*iutdif + (param(3) - '0')
        is=4
      endif
      iutdif=iutdif*isign
   11 if(param(is) .eq. 'D') then
         idst=1
      else
         idst=0
      endif
c
c     if port was not specified above then read it in
c     now
c
      if(iport(:1) .eq. ' ') then
        read(2,97), il,iport(:il)
   97   format(q,a<il>)
      endif
c
      close(unit=2,dispose='save')
c
      if(idebug .ne. 0) then
      type 18,ilist,ichk,iarc,iset,idebug,irate,
     +  iport,telno,iutdif,idst
   18 format(' Operations Requested (1=on, 0=off)'/
     + ' List ='I2' Check='I2' Archive='I2' Set='I2' Debug='I2/
     + ' Rate ='I2/
     + ' Port ='a20,' Telephone Number='40A1/
     + ' UTC offset='I5' Daylight Savings Flag='I2)
      endif
c
c    allocate port and assign a channel to it
c    note that channel number is stored and defined in iochan2.inc
c    include file
c
      istat=sys$alloc(iport,,,,)
      if(istat .ne. 1) then
        type 1901,istat
 1901   format(' Allocate port failed, stat='i8)
        call lib$stop(%val(istat))
      endif
      istat=sys$assign(iport,ichnl2,,)
      if(istat .ne. 1) then
        type 1900,istat
 1900   format(' assign port failed, stat='i8)
        call lib$stop(%val(istat))
      endif
c
c     now read current characteristics of port
c
      istat=sys$qiow( %val(1),%val(ichnl2),%val(io$_sensemode),
     + iosb2,,,%ref(ibb),%VAL(8),,,,)
      if(istat .ne. 1) then
        type 1904,istat
 1904   format(' Error in sense characteristics, stat='i8)
        call lib$stop(%val(istat))
      endif
      if(idebug .ne. 0) type 1905,ibb
 1905 format(' current characteristics='8i5)
c    now set port to 1200 baud/no parity
c
      istat=sys$qiow( %val(1),%val(ichnl2),%val(io$_setmode),
     + iosb2,,,%ref(ibb),%val(8),%val(TT$C_BAUD_1200),
     + %val(0),,)
      if(istat .ne. 1) then
        type 1902,istat
 1902   format(' Error in set characteristics, stat='i8)
        TYPE 1903,IOSB2
 1903   FORMAT('IOSB='4I7)
        call lib$stop(%val(istat))
      endif
c
c     if rate estimate to be made, get last value from file now
c
  133 if(irate .ne. 0) call getlst
c
c    if first digit is m then manual dialing is requested
c    print message and wait
c    otherwise call dialing subroutine
c
      telno(1)= telno(1) .and. '177'o
      if(telno(1) .gt. '140'o) telno(1)=telno(1) - '40'o
      IF(TELNO(1) .EQ. 'M')  then
        type 123
  123   format(' Dial telephone now. Enter'/
     +   '       y<cr> when connection is complete'/
     +   '       n<cr> if busy or no answer.')
        type 125
  125   format('$Ans=')
        read(5,124) ians
  124   format(a1)
        IANS=IANS .AND. '177'O
        if(ians .gt. '140'o) ians= ians -'40'o
        if(ians .eq. 'N') stop 'No answer'
      else
        call dialit(telno,istat)
        if(istat .eq. 0) stop 'No answer'
      endif
      call rcvdat
      call stoptr
      call hangup
      call exit
      end
      subroutine rcvdat
c
c     receives data from the nist acts service and stores the
c     lines in a buffer.  up to 40 lines are received
c
      byte ilen(20),lines(80,20),ijj
c
      common/ops/ ilist,ichk,iarc,iset,idebug,irate
      integer*2 iosb3(4)
c
      include 'iochan2.inc'
c
c     initialize parse/set status flag
c
      mmstat=0
c
      do 1 ilin=1,20
c
      ijjk=0
 9876 istat=sys$qiow( %val(1), %val(ichnl2),%val(iornt),
     +  iosb2,,,
     +  %ref(IJJ),%val(1),%val(10),,,)
d     type 6262,istat,ijj,iosb2
d6262 format(' istat='i8' char='a1' iosb2='4i5)
C
C     TERMINATE READ LOOP ON A TIMEOUT
C
      if(iosb2(1) .eq. SS$_TIMEOUT) go to 2
C
C     IGNORE ALL OTHER ERRORS
C
      if(iosb2(1) .ne. 1) go to 9876
      IJJ=IJJ .AND. '177'O
      IF( (IJJ .EQ. '15'O) .or. (iosb2(3) .eq. 13) .or.
     +    (ijjk .gt. 75) ) THEN
        if(ijjk .eq. 0) go to 9876    !ignore empty lines
        ILEN(ILIN)=IJJK
D       WRITE(4,5252) (LINES(MM,ILIN),MM=1,IJJK)
D5252   FORMAT(1X,80A1)
        GO TO 1
      ENDIF
      IF(IJJ .LT. '40'O) GO TO 9876
      IJJK=IJJK+1
      LINES(IJJK,ILIN)=IJJ
c
c     if this is a line terminator then echo it back out
c     to nist to measure the line delay
c
      IF( (IJJ .EQ. '*') .OR. (IJJ .EQ. '#') ) THEN
      istat=sys$qio( %val(2),%val(ichnl2),%val(iowal),
     +iosb3,,,
     + %ref(ijj),%val(1),,%val(0),,)
      ENDIF
      IF( IJJ .EQ. '#') CALL PARSET(LINES(1,ILIN),MMSTAT)
      IF(MMSTAT .EQ. 0) GO TO 9876
      ILEN(ILIN)=IJJK
      GO TO 33
    1 CONTINUE
      ILIN=21
    2 ILIN=ILIN-1
   33 IF(ILIST .NE. 0) THEN
      DO 5544 I=1,ILIN
      MM=ILEN(I)
      TYPE 5545, (LINES(NN,I),NN=1,MM)
 5545 FORMAT(1X,80A1)
 5544 CONTINUE
      ENDIF
      RETURN
      END
      SUBROUTINE PARSET(BUF,MMSTAT)
C
C     RECEIVES A LINE FROM NIST AND PARSES IT TO EXTRACT TIME
C     IF SECOND IS TOO NEAR END OF MINUTE, WAIT FOR NEW MINUTE
C     CHECKS THAT TIME OF TWO CONSECUTIVE LINES DIFFER BY
C     EXACTLY ONE SECOND
C
      COMMON/OPS/ILIST,ICHK,IARC,ISET,IDEBUG,IRATE
C
C     COMMON BLOCK LVAL HOLDS PREVIOUS VALUES READ FROM ARCHIVE FILE
C     FOR RATE ESTIMATE IF REQUESTED
C
      COMMON/LVAL/IPYR,IPMO,IPDY,IPHR,IPMIN,IPSEC,PDIFF,CUNIT
      BYTE CUNIT
C
      REAL*8 SDIFF
      DATA imo / 0 /       !month = 0 signals 1st time through
      BYTE BUF(80)
      INTEGER*2 ISYS(4),JSYS(7)
      INTEGER*2 MON(12)
      DATA MON /0,31,59,90,120,151,181,212,243,273,304,334/
C
C     THE FOLLOWING ARE USED TO PRODUCE THE ASCII TIME
C     STRING NEEDED TO SET THE TIME
C
      CHARACTER*3 NMON(12)
      DATA NMON /'JAN','FEB','MAR','APR','MAY','JUN',
     + 'JUL','AUG','SEP','OCT','NOV','DEC'/
      CHARACTER*23 KSYS
      include '($syssrvnam)'
c
c     following array holds the binary versions of the
c     received characters
c
      integer*2 mc(14)
C
C     MMSTAT SETS RETURN STATUS.  IF MMSTAT=0 CONTINUE, =1 FINISHED
C
      MMSTAT=0
      DO 1 I=1,80
      IF(BUF(I) .EQ. '-') GO TO 2
    1 CONTINUE
      imo=0           !line has syntax error, start over next time
      RETURN 
    2 mc(1) =buf(i-2) - '0'    !year, high digit
      mc(2) =buf(i-1) - '0'    !year, low digit
      mc(3) =buf(i+1) - '0'    !month, high digit
      mc(4) =buf(i+2) - '0'    !month, low digit
      mc(5) =buf(i+4) - '0'    !day, high digit
      mc(6) =buf(i+5) - '0'    !day, low digit
      mc(7) =buf(i+7) - '0'    !hour, high digit
      mc(8) =buf(i+8) - '0'    !hour, low digit
      mc(9) =buf(i+10)- '0'    !minute, high digit
      mc(10)=buf(i+11)- '0'    !minute, low digit
      mc(11)=buf(i+13)- '0'    !second, high digit
      mc(12)=buf(i+14)- '0'    !second, low digit
      mc(13)=buf(i+16)- '0'    !dst, high digit
      mc(14)=buf(i+17)- '0'    !dst, low digit
      do 5 i=1,14
        mx=mc(i)
        if( (mx .lt. 0) .or. (mx .gt. 9) ) then   !character is not a digit
          imo=0         !error here, start over again next time
          return
        endif
    5 continue
      IF(imo .EQ. 0) THEN
        IYR=10*mc(1) + mc(2)
        IMO=10*mc(3) + mc(4)
        IDY=10*mc(5) + mc(6)
        IHR=10*mc(7) + mc(8)
        IMN=10*mc(9) + mc(10)
        ISC=10*mc(11)+ mc(12)
        IDS=10*mc(13)+ mc(14)
        IF(IDEBUG .NE. 0) TYPE 3, IYR,IMO,IDY,IHR,IMN,ISC,IDS
    3 FORMAT(7I4)
C
C     IF NEAR END OF MINUTE, SET imo =0 SO THIS PORTION
C     WILL BE REPEATED IN NEW MINUTE
C
        IF(ISC .GT. 57) imo=0
        RETURN
      ENDIF
C
C    IF WE GET HERE THIS MUST BE SECOND TIME THROUGH
C    GET THIS TIME AND COMPARE WITH PREVIOUS LINE
C
      JYR=10*mc(1) + mc(2)
      JMO=10*mc(3) + mc(4)
      JDY=10*mc(5) + mc(6)
      JHR=10*mc(7) + mc(8)
      JMN=10*mc(9) + mc(10)
      JSC=10*mc(11)+ mc(12)
      JDS=10*mc(13)+ mc(14)
      IF(IDEBUG .NE. 0) TYPE 3, JYR,JMO,JDY,JHR,JMN,JSC,JDS
C
C    IF TWO LINES NOT CONSECUTIVE SET IYR=0 SO ENTIRE PROCESS
C    WILL BE REPEATED WITH NEXT TWO LINES
C
      IF(  (IYR .NE. JYR)  .OR.
     +     (IMO .NE. JMO)  .OR.
     +     (IDY .NE. JDY)  .OR.
     +     (IHR .NE. JHR)  .OR.
     +     (IMN .NE. JMN)  .OR.
     +     (IDS .NE. JDS)  .OR.
     +     (ISC .NE. (JSC -1))  ) THEN
      IYR = 0
      RETURN
      ENDIF
C
C     IF CHECK OR ARC IS ENABLED, THEN 
C     GET SYSTEM TIME FOR COMPARISONS
C      
C
      istat=sys$gettim(%ref(isys))
      if(istat .ne. 1) call lib$stop(%val(istat))
      istat=sys$numtim(%ref(jsys),%ref(isys))
      if(istat .ne. 1) call lib$stop(%val(istat))
c
c	note that the system returns a full 4-digit year.
c
      if(idebug .ne. 0) type 2345,jsys
 2345 format(' current time='7i6)
C
C     CONVERT RECEIVED TIME TO LOCAL TIME USING DAYLIGHT SAVINGS
C     TIME FLAG IF REQUESTED.  THIS MUST BE DONE BEFORE SYSTEM CAN
C     BE SET OR TIME COMPARED
C
C     NOTE THAT CONVERSION TO LOCAL TIME CANNOT CHANGE MINUTE OR
C     SECOND AND THAT THEY ARE CORRECT NOW
C
      CALL CVTLCL(JYR,JMO,JDY,JHR,JDS)
C
C     IF SET ENABLED THEN SET THE TIME NOW
c     convert the two digit year received from the 
c     ACTS system into a 4-digit year.
c
      if(jyr .ge. 80) then 
         jyr= jyr + 1900
      else
         jyr=jyr  + 2000
      endif
C
      IF(ISET .NE. 0) THEN
      ENCODE(23,2323,KSYS) JDY,NMON(JMO),JYR,JHR,
     + JMN,JSC
 2323 FORMAT(I2,'-',A3'-',I4.4,1X,I2.2,':',
     + I2.2,':',I2.2,'.00')
      ISTAT=SYS$BINTIM(KSYS,ISYS)
      IF(ISTAT .NE. 1) CALL LIB$STOP(%VAL(ISTAT))
      ISTAT=SYS$SETIME(ISYS)
      IF(ISTAT .NE. 1)CALL LIB$STOP(%VAL(ISTAT))
      ENDIF
C
C     NOW DEAL WITH COMPARING NIST AND SYSTEM TIMES
C
C     IF ICHK, IARC, IRATE ARE ZERO, THEN WE ARE FINISHED.
C
      IF( (ICHK .EQ. 0) .AND. 
     +    (IARC .EQ. 0) .AND.
     +    (IRATE .EQ. 0) )GO TO 77
C
C    GET ELAPSED TIME SINCE LAST COMPARISON IF RATE ESTIMATE REQUESTED
C
      IF(IRATE .NE. 0) THEN
      ELAPSE=365.*(JYR - IPYR) + MON(JMO) - MON(IPMO)
      IF( ( ( (JYR-1900)  .AND. 3) .EQ. 0) .AND. 
     +    (  JMO          .GT. 2) ) ELAPSE=ELAPSE + 1.
      IF( ( ( (IPYR-1900) .AND. 3) .EQ. 0) .AND.
     +    (  IPMO         .GT. 2) ) ELAPSE=ELAPSE - 1.
      ELAPSE=ELAPSE + (JDY - IPDY) +
     +                (JHR - IPHR)/24.D+0 + 
     +                (JMN - IPMIN)/1440.D+0 +
     +                (JSC - IPSEC)/86400.D+0
      IF(CUNIT .EQ. 'D') THEN
        SDIFF=86400.D+0*PDIFF
      ELSEIF(CUNIT .EQ. 'H') THEN
        SDIFF=3600.D+0*PDIFF
      ELSEIF(CUNIT .EQ. 'M') THEN
        SDIFF=60.D+0*PDIFF
      ELSEIF(CUNIT .EQ. 'S') THEN
        SDIFF=PDIFF
      ELSEIF(CUNIT .EQ. 'T') THEN
        SDIFF=PDIFF/ISYS(8)
      ENDIF
      IF(IDEBUG .NE. 0) TYPE 5555,ELAPSE,SDIFF
 5555 FORMAT(1X,F9.4' Days since last comparison which was 'F8.3' sec.')
      ENDIF
      ifract=jsys(7)
      idiff=jsys(1) -jyr
      IF( ABS(IDIFF) .GT. 1) THEN
      QDIFF=IDIFF
      MDIFF='yr'
      IF(IRATE .NE. 0)SDIFF=(31536000.D+0*QDIFF - SDIFF)/ELAPSE
      GO TO 47
      ENDIF
      III=jsys(2)
      IDIFF=365*IDIFF + MON(III) - MON(JMO) + jsys(3) - JDY
      IF( ((JYR .AND. 3) .EQ. 0)  .AND. (JMO .GT. 2))IDIFF=IDIFF-1
      IF( ((jsys(1).AND.3).EQ.0) .AND. (jsys(2) .GT.2))IDIFF=IDIFF+1
      IF( ABS(IDIFF) .GT. 1) THEN
      QDIFF=IDIFF
      MDIFF='dy'
      IF(IRATE .NE. 0) SDIFF=(86400.D+0*QDIFF - SDIFF)/ELAPSE
      GO TO 47
      ENDIF
      IDIFF=24*IDIFF + jsys(4) - JHR
      IF( ABS(IDIFF) .GT. 10) THEN
      QDIFF=IDIFF
      MDIFF='hr'
      IF(IRATE .NE. 0) SDIFF=(3600.D+0*QDIFF - SDIFF)/ELAPSE
      GO TO 47
      ENDIF
      IDIFF=60*IDIFF + jsys(5) - JMN
      IF( ABS(IDIFF) .GT. 100) THEN
      QDIFF=IDIFF
      MDIFF='m '
      IF(IRATE .NE. 0) SDIFF=(60.D+0*QDIFF - SDIFF)/ELAPSE
      GO TO 47
      ENDIF
      IDIFF=60*IDIFF + jsys(6) - JSC
      QDIFF=FLOAT(IDIFF) + FLOAT(IFRACT)/100.
      MDIFF='s '
      IF(IRATE .NE. 0) SDIFF=(QDIFF - SDIFF)/ELAPSE
   47 IF(ICHK .NE. 0) THEN
      TYPE 44,(jsyS(I),I=1,6),IFRACT
   44 FORMAT(' System Time='I4'-'I2.2'-'I2.2,1X,I2.2':'I2.2':'I2.2'.'
     +  I2.2)
      TYPE 45,JYR,JMO,JDY,JHR,JMN,JSC
   45 FORMAT(' NIST Time  ='I4'-'I2.2'-'I2.2,1X,I2.2':'I2.2':'I2.2,
     + '.00')
      IF(QDIFF .GE. 0.01) THEN
      TYPE 48,QDIFF,MDIFF
      ELSEIF(QDIFF .LE. -0.01) THEN
      TYPE 49,-QDIFF,MDIFF
      ELSE
      TYPE 50
      ENDIF
   48 FORMAT(' System time fast by 'F6.2,A2)
   49 FORMAT(' System time slow by 'F6.2,A2)
   50 FORMAT(' System time correct to within 10 msec.')
      ENDIF
      IF(IRATE .NE. 0) THEN
      TYPE 5554,SDIFF
 5554 FORMAT(' Approximate rate offset='F8.3' s/day.')
      ENDIF
      IF(IARC .NE. 0) THEN
      OPEN(UNIT=2,NAME='NISTIME.DIF',TYPE='UNKNOWN',
     + FORM='FORMATTED',ACCESS='APPEND',CARRIAGECONTROL='LIST',
     + DISPOSE='SAVE')
      WRITE(2,51) JYR,JMO,JDY,JHR,JMN,JSC,QDIFF,MDIFF
   51 FORMAT(1X,SS,I4'-'I2.2'-'I2.2,1X,I2.2':'I2.2':'I2.2,1X,
     + SP,F7.2,A2)
      CLOSE(UNIT=2,DISPOSE='SAVE')
      ENDIF
   77 MMSTAT=1
      RETURN
      END
      SUBROUTINE CVTLCL(JYR,JMO,JDY,JHR,JDS)
C
C     CONVERT RECEIVED TIME TO LOCAL TIME BASED ON CONSTANTS IN
C     COMMON BLOCK /LOCAL/
C
      COMMON/LOCAL/IUTDIF,IDST
C
C    FOLLOWING ARRAY HOLDS LAST DAY OF EVERY MONTH
C
      INTEGER*2 LDAY(12)
      DATA LDAY /31,28,31,30,31,30,31,31,30,31,30,31/
C
C     FEB HAS 29 DAYS IN A LEAP YEAR
C
      IF( (JYR .AND. 3) .EQ. 0) LDAY(2)=29
C
C     FIRST CONVERT HOURS TO LOCAL TIME
c     => note that the conversion assumes standard time
C
      JHR=JHR + IUTDIF
C
C     ADD DAYLIGHT SAVINGS TIME IF REQUESTED AND IF NEEDED
C
C     IF IDST NON ZERO THEN DST WAS REQUESTED
C     IF JDS IS 2 - 50 INCLUSIVE WE ARE SOLIDLY IN DST
C
C     FLAG KDS SHOWS THAT CONVERSION TO DST HAS BEEN DONE
C
      IF( (IDST .NE. 0)  .AND.
     +    (JDS  .LE.50)  .AND.
     +    (JDS  .GT. 1)  ) THEN
         JHR= JHR + 1
         KDS=1
      ELSE
         KDS=0
      ENDIF
c
c     if conversion to dst was requested and this is the
c     Fall transition day and the transition time has not 
c     yet arrived, then we are still on dst.  note that
c     the test uses jhr < 1 even though the transition is
c     at 0200 since jhr has been converted to standard time
c     above so that the transition time will arrive when jhr=1
c
      if( (idst .ne. 0) .and.
     +    (jds  .eq. 1) .and.
     +    (jhr  .lt. 1) ) then
        jhr=jhr + 1
        kds=1
      endif
C
C    NOW SEE IF CHANGING THE HOUR AFFECTED THE DAY
C
C    IF HOUR IS NEGATIVE, GO BACK TO PREVIOUS DAY
C    AND THEN PROPAGATE BACKWARDS THROUGH MONTH AND YEAR IF NEEDED
C
C     SINCE THE DAYLIGHT SAVINGS TIME FLAG IS LINKED TO LOCAL TIME
C     A CHANGE IN DAY/DATE MEANS A CHANGE IN FLAG.  THIS IS ONLY
C     IMPORTANT IF WE ARE IN FACT ON A TRANSITION DAY.
C
      IF(JHR .LT. 0) THEN
         JHR=JHR + 24
         JDY=JDY - 1
         JDS=JDS + 1
         IF(JDY .LT. 1) THEN
            JMO=JMO - 1
            IF(JMO .LT. 1) THEN
               JMO=12
               JYR=JYR -1
            ENDIF
            JDY=LDAY(JMO)
         ENDIF
      ENDIF
      IF(JHR .GT. 23) THEN
         JHR=JHR - 24
         JDY=JDY + 1
         JDS=JDS - 1
         IF(JDY .GT. LDAY(JMO)) THEN
            JDY=1
            JMO=JMO + 1
            IF(JMO .GT. 12) THEN
               JMO=1
               JYR=JYR + 1
            ENDIF
         ENDIF
      ENDIF
C
C    IF DST CONVERSION REQUESTED AND IF IT WAS NOT DONE BEFORE
C    THEN NOW CHECK FOR CONVERSION ON TRANSITION DAYS  --
C    AFTER 2 AM ON THE DAY DST STARTS AND BEFORE 2 AM ON THE DAY
C    IT ENDS
c    since jhr is set above to standard time by default, the
c    Fall conversion will actuall arrive at jhr=1
C
      IF( (IDST .NE. 0) .AND. (KDS .EQ. 0) ) THEN
         IF( (JDS .EQ. 51) .AND. (JHR .GE. 2) ) JHR=JHR + 1
         IF( (JDS .EQ.  1) .AND. (JHR .LT. 1) ) JHR=JHR + 1
      ENDIF
      RETURN
      END
      SUBROUTINE DIALIT(TELNO,istat)
C
C     THIS SUBROUTINE DIALS A NUMBER USING HAYES COMMANDS
C     THE DIALER IS CONNECTED VIA LUN ichnl2
c    which is defined in iochan2.inc
C     
C     THIS SUBROUTINE ASSUMES THAT THE CALLING TASK HAS
C     PROPERLY SET THE SPEED AND PARITY OF THE LINE CONNECTED TO
C     THE DIALER.
C
C
      BYTE BUFI(80),BUFO(80)
      BYTE IZ(4),TELNO(40)
C
C     IZ IS MODEM INTIALIZE STRING
C
      DATA IZ / 'A','T','Z','15'O/
C
C     IOPT IS THE OPTION COMMAND STRING FOR THE MODEM
C
      BYTE IOPT(12)
      DATA IOPT/ 'A','T',' ','E','1',' ','Q','0',' ','V','0','15'O/
C
      include 'iochan2.inc'
C
C     SEND RESET STRING TO MODEM
C
      ISTAT=SYS$QIOW(%VAL(1),%VAL(ICHNL2),%VAL(IOWAL),
     + IOSb2,,,
     + %REF(IZ),%VAL(4),,%VAL(0),,)
      if(istat .ne. 1) then
        type 1900,iosb2
 1900   format('status block='4i6)
        call lib$stop(%val(istat))
      endif
C
C    WAIT FOR MODEM TO FINISH RESET
C
      CALL WAIT(2)
C
C     NOW SET MODEM CHARACTERISTICS
C
      ISTAT=SYS$QIOW(%VAL(1),%VAL(ICHNL2),%VAL(IOWAL),
     + IOSb2,,,
     + %REF(IOPT),%VAL(12),,%VAL(0),,)
      IF(ISTAT .NE. 1) CALL LIB$STOP(%VAL(ISTAT))
      CALL WAIT(2)
C
      BUFO(1)='A'
      BUFO(2)='T'
      BUFO(3)='D'
      IJ=3
      DO 2 I=1,40
      IF( TELNO(I) .EQ. 0) GO TO 22
      IJ=IJ+1
      BUFO(IJ)=TELNO(I)
    2 CONTINUE
   22 ij=ij+1
      bufo(ij)='15'o
      istat=sys$qiow(%val(1),%val(ichnl2),%val(iowal),
     + iosb2,,,
     + %ref(bufo),%val(ij),,%val(0),,)
       if(istat .ne. 1) call lib$stop(%val(istat))
c
c     wait 2 seconds for modem to receive string then
c     clear input buffer and listen for response
c
      call wait(2)
  800 istat=sys$qiow( %val(1),%val(ichnl2),
     + %val(iornt), iosb2,,,
     + %ref(bufi),%val(70),%val(100),%val(0),,)
      if(istat .ne. 1) then
        type 1904,istat,iosb2
 1904   format(' read error, status='i8/
     +         ' iosb='4i6)
        call lib$stop(%val(istat))
      endif
C
C      IF THE RESPONSE READ BY THIS QIO IS 3,4,6,7 OR 8, THEN THE CALL DID
C      NOT GO THROUGH -- WE HAVE A FATAL ERROR.  IF THE RESPONSE
C      IS 1,5, OR 10, WE HAVE A LIFT OFF.  OTHER RESPONSES ARE
C      INTERMEDIATE ONES OR ARE RESIDUES OF THE DIALING COMMANDS.
C      IN THAT CASE GO BACK AND READ THE NEXT LINE
C
      ILIN=ILIN+1
C  
C     POSSIBLE INTERMEDIATE RESPONSES ARE 0 OR 2 OR A NON-DIGIT.
C     IN THESE CASES GO BACK AND READ THE NEXT RESPONSE
C
      IF( (BUFI(1) .LT. '0') .OR. (BUFI(1) .GT. '9') .OR.
     +    (BUFI(1) .EQ. '0') .OR. (BUFI(1) .EQ. '2') ) GO TO 800
C
C     ANY OTHER RESPONSE EXCEPT ON LINE IS BAD NEWS
C
      IF(iosb2(2) .GT. 0)
     +TYPE 888,(BUFI(K),K=1,iosb2(2))
  888 FORMAT(' MODEM ANSWER='40A1)
      ISTAT=0
      IF( (BUFI(1) .EQ. '1') .OR. (BUFI(1) .EQ. '5') )istat=1 
      return
      END
      SUBROUTINE HANGUP
C
C     HANGS UP THE DIALER.
C
      BYTE IQUIT(3),IHANG(4)
C
      DATA IQUIT / '+','+','+' /
      DATA IHANG / 'A','T','H','15'O /
C
      INCLUDE 'IOCHAN2.INC'
C
      CALL WAIT(3)
      ISTAT=SYS$QIOW( %VAL(1),%VAL(ICHNL2),%VAL(IOWAL),
     + IOSB2,,,
     + %REF(IQUIT),%VAL(3),,%VAL(0),,)
      IF(ISTAT .NE. 1) CALL LIB$STOP(%VAL(ISTAT))
      CALL WAIT(3)
      ISTAT=SYS$QIOW(%VAL(1),%VAL(ICHNL2),%VAL(IOWAL),
     + IOSB2,,,
     + %REF(IHANG),%VAL(4),,%VAL(0),,)
      IF(ISTAT .NE. 1) CALL LIB$STOP(%VAL(ISTAT))
c
c     read characters until there is nothing more to read
c     the read will exit on a time out in this case.
c
      do 55 i=1,100                 !maximum of 100 glitch chars.
      istat=sys$qiow(%val(2),%val(ichnl2),%val(iornt),
     +  iosb2,,,
     +  %ref(ijj),%val(1),%val(3),,,)
      if(iosb2(1) .eq. SS$_TIMEOUT) return    !that's all folks
   55 continue
      RETURN
      END
      SUBROUTINE STOPTR
C
C     SEND COMMAND TO STOP NIST TRANSMISSIONS
C
      BYTE ISTP 
C
      DATA ISTP / '%' /
C
      INCLUDE 'IOCHAN2.INC'
C
      ISTAT=SYS$QIOW( %VAL(1),%VAL(ICHNL2),%VAL(IOWAL),
     + IOSB2,,,
     + %REF(ISTP),%VAL(1),,%VAL(0),,)
      IF(ISTAT .NE. 1) CALL LIB$STOP(%VAL(ISTAT))
      CALL WAIT(3)
      RETURN
      END
      SUBROUTINE GETLST
C
C     THIS SUBROUTINE READS THE ARCHIVE FILE AND GETS THE
C     LAST VALUE STORED THERE FOR SUBSEQUENT ESTIMATE OF 
C     THE RATE OFFSET OF THE COMPUTER CLOCK
C
      COMMON/OPS/ILIST,ICHK,IARC,ISET,IDEBUG,IRATE
      COMMON/LVAL/IPYR,IPMO,IPDY,IPHR,IPMIN,IPSEC,PDIFF,CUNIT
      BYTE CUNIT
      OPEN(UNIT=2,NAME='NISTIME.DIF',TYPE='OLD',ACCESS='SEQUENTIAL',
     + FORM='FORMATTED',CARRIAGECONTROL='LIST',DISPOSE='SAVE',
     + READONLY)
      INUM=0
    1 READ(2,2,END=3,ERR=5)IPYR,IPMO,IPDY,IPHR,IPMIN,IPSEC,
     + PDIFF,CUNIT
    2 FORMAT( 1x,i4,5(1X,I2),F8.2,A1)
      INUM=INUM + 1
      GO TO 1
    5 TYPE 6,INUM
    6 FORMAT(' GETLST -- Error reading archive file at line'I5)
      IARC=0
      IRATE=0
      RETURN
    3 IF(IDEBUG .NE. 0) TYPE 7,INUM,IPYR,IPMO,IPDY,IPHR,IPMIN,IPSEC,
     +   PDIFF,CUNIT
    7 FORMAT(' Last values read from file at line' I2'=',
     + 6(1X,I4),F7.2,A1)
C
C    CONVERT UNITS TO UPPER CASE IF NECESSARY
C
      IF(CUNIT .GT. '140'O) CUNIT= CUNIT - '40'O
      CLOSE(UNIT=2)
      RETURN
      END
      SUBROUTINE WAIT(J)
C
C     DELAYS EXECUTION OF CALLER BY J SECONDS
C
      INCLUDE '($SYSSRVNAM)'
      INTEGER*4 IV(2)
      CHARACTER*23 IBUF
C
      IF(J .GT. 99) THEN
        TYPE 2,J
    2   FORMAT(1X,I5's too big, max delay=99')
        STOP 'ERROR IN WAIT'
      ENDIF
      ENCODE(23,1,IBUF) J
    1 FORMAT(' 0 ::'I2)
      ISTAT=SYS$BINTIM(IBUF,IV)
      IF(ISTAT .NE. 1) CALL LIB$STOP(%VAL(ISTAT))
      ISTAT=SYS$SCHDWK( , ,IV(1),)
      IF(ISTAT .NE. 1) CALL LIB$STOP(%VAL(ISTAT))
      ISTAT=SYS$HIBER()
      IF(ISTAT .NE. 1) CALL LIB$STOP(%VAL(ISTAT))
      RETURN
      END
