/*
        include file getdif.h

        this file is used to define the constants
        that govern the compilation of subroutine getdif
	and the other subroutines that it calls to 
	implement a simplified version of the ACTS
	protocol

        if the constant IBMPC is defined then the
        program is intended to run on an IBM-PC
        (or equivalent machine) under MSDOS.

        if the constant SUN is defined then the
        program is intende to run on a SUN work
        station.  This version is as non-specific
        as possible so that it should run with only
        minor modifications in other unix environments.
*/
/*	#define IBMPC  1   */
#define SUN   1
/*
        There are two MSDOS versions of the program:

        if the constant BIOS is defined, then the
        program will access the RS-232 port via calls
        to the system BIOS.

        if the constant BIOS is not defined, then the
        program uses direct input/output via inport and outport.

        if the following line is not a comment, then input/output
        will be done via calls to the BIOS.
*/
/*		#define BIOS 1     */
/*
        if IBMPC is defined, then an MSDOS version is being
        generated.  In that case, if direct input/output is
        to be used, the following constants define the
        offsets from the port address for the various
        registers of the rs-232 device.
*/
#ifdef IBMPC
#ifndef BIOS
#define txbuf  0        /* transmit buffer offset */
#define rxbuf  0        /* receiver buffer offset */
#define divlsb 0        /* divisor latch, low part */
#define divmsb 1        /* divisor latch, high part */
#define intena 1        /* interrupt enable register */
#define intid  2        /* interrupt ident. register */
#define lcreg  3        /* line control register */
#define modreg 4        /* modem control register */
#define lsreg  5        /* line status register   */
#define msreg  6        /* modem status register  */
#endif
#endif
/*
	if this is the UNIX version then parameter tickus
	gives the time interval between ticks in microseconds.
	typical values are 20 milliseconds for a SUN,
	10 milliseconds for a VAX and 3906 milliseconds for
	a DEC ULTRIX/RISC machine.
	this value is used in the interpolation routines
	interp.c, arcdif.c and parset.c.  If this parameter
	is specified as 0 then the interpolation between ticks
	is disabled and the comparison will use the integer 
	tick value as read from the system clock.
	the constant INTERP must be defined as 1 if interplation
	is to be used.  this additional parameter speeds up the 
	tests if interolation is to be disabled.
	this constant is not needed in the MS-DOS environment
	since all of those machines use a constant value of
	18.2 ticks/second or about 55 milliseconds between ticks.

*/
#ifdef SUN
#define INTERP 0
#include "sizint.h"
LONG tickus = 3906;
#endif
/*
	the following constants are usually set using the ACTS
	configuration file. they are defined here because they
	almost never change and so that subroutine getdif does 
	not have to open a configuration file each time it it called.

	The strings number and alt_number are the ACTS telephone
	number and a backup number to be used if the first one
	is busy or does not answer.  The first number is usually
	the base of the rotary for low-speed ACTS while the
	second is the standard ACTS rotary, which has more lines
	but is likely to have a larger asymmetry.
	
	The parameters L0M0 turn the speaker off all the time.

	The program will switch to the alternate number if there
	are three consecutive retry failures with the primary
	number.
*/
char *number = "ATDT9,4942668\r";	/*Lo-speed ACTS telephone number*/
char *alt_number="ATDT9,4944774\r";	/*standard ACTS number */
#ifdef SUN
char *dv     = "/dev/tty00";		/*serial port to use */
#endif
#ifdef IBMPC
int utcdif = -7;   /* local standard time - utc in hours */
int dsflag = 1;    /* daylight savings time? 1 = yes, 0 = no */
int atflag = 1;    /* AT-type machine? 1=yes, 0=no */
#endif
int echo   = 1;    /* expect echoes from modem? 1=yes, 0=no  */
int hs     = 1;    /* 1200 baud dialing? 1=yes, 0=no, use 300*/
int retry  = 6;	   /* number of times to retry if busy/no answer */
extern int cmport;	   /*0 for com1 port, 1 for com2 port, etc.  */
extern int cmadr;	   /*address of com port if PC or file handle if SUN */
