#include <stdio.h>
#include <ctype.h>
#include "sizint.h"
void main()
/*
	program sha-leapsec calculates the sha signature for the
	file specified as the standard input. It is normally used
	to calculate the hash of the leap second file in the ftp
	directory, which is named leap-seconds.list.
	Although the program uses the standard algorithm specified
	in FIPS 180-1, the algorithm is applied only to a portion
	of the leap second file. Specifically:

	1. comment lines, which start with # are not included
	   when a comment signal is read, the comment begins at that
	   point and continues to the end of the line.
	2. white space characters in non-comment lines are not
	   included.
	3. the numerical values (but not the white space) in
  	   the two special lines #$ and #@ are included.
*/
{
void init_sha(void);
void sha(ULONG w[16],ULONG h[5]);
ULONG w[16];
extern ULONG h[5];	/*hash computed by the program*/
ULONG r[5]={0,0,0,0,0};	/*hash read from the file*/
int i,j;
char cc;		/*the current input character*/
int icomment= 0;	/*flag to show comment line status*/
int icount= 0;		/*number of bits in the message*/
int iword=0;		/*current word*/
int ibyte=0;		/*current byte*/
	init_sha();			/*initialize constants*/
	for(i=0; i<16; i++) w[i]=0;	/*initialize array*/
rdnxtchar:		/*come here to read the next character*/
	cc=getchar();	/*read the next character from the standard input*/
	if(cc == '\n') 
	   {
	   icomment= 0;		/*comment ends with newline*/
	   goto rdnxtchar;	/*read the next character*/
	}
	if(cc == EOF)	/*end of the file*/
	    {
/*
	    the first step now is to add the trailing character,
	    which is hex 80 and fill the rest of this word with
	    0s. the remaining words of the array are to be filled
	    with 0, except for the last word, which has the
	    bit count. note that the entire array was set to 0
	    during the initialization and so need not be done 
	    here.
*/
	   w[iword] <<= 8;      /*left shift the stuff already there*/
	   w[iword] |= 0x80;    /*add the trailing character*/
	   while(++ibyte < 4) w[iword] <<= 8;
	   if(++iword == 16)  	/*the array is full*/
	      {
	      sha(w,h);		/*compute hash of another block*/
	      for(i=0; i<16; i++) w[i]= 0;
	   }
/*
	   add the bit count to the end of the w array and compute
	   the last partial hash. if the real stuff above ended on
	   an exact array boundary then this computation will be for
	   an array of all 0s.
*/ 
	   w[15]= icount << 3;		/*convert byte count to bits*/
	   sha(w,h);
	   printf("\n the digest of the input is \n");
           for(i=0; i<5; i++) printf(" %x",h[i]);
	   printf("\n the digest read from the file is \n");
	   for(i=0; i<5; i++) printf(" %x",r[i]);
	   for(i=0; i<5; i++) if(h[i] != r[i])	
		{
		printf("\n digests differ.\n");
		exit(0);
	   }
	   printf("\n digests agree.\n");
	   exit(0);	/*that's all folks*/
	}
	if(icomment == 2) goto rdnxtchar;	/*ignore comments*/
	if( (icomment == 0) && (cc == 043) )	/*possible new comment*/
	    {
	    icomment= 1;		/*check next character*/
	    goto rdnxtchar;
	}
/*
	if the previous character was a # then there are two special
	cases where the values are part of the checksum. They are
	#$ and #@. Treat these two lines as regular data lines.
	there is a third special case #h which signals that the
	values on the line are a hash value in hex. read this 
	stuff and compare it later to the hash that we compute.
	note that the hash value line itself is not counted in the
	hash and is treated as a comment.  Anything else is a real 
	comment.
*/
	if(icomment == 1)		/*previous character was # */
	   {
	   if(cc == 0150)		/*hash value on this line*/
	      {
	      j=scanf(" %x %x %x %x %x",&r[0],&r[1],&r[2],&r[3],&r[4]);
	      if(j != 5) fprintf(stderr,"\n format error in hash value.");
	   }				/*end of hash value on this line*/
	   if( (cc == 044) || (cc == 0100))	icomment= 0;
	   else  				icomment= 2;
	   goto rdnxtchar;		
	}			/*end of previous character was # */
/*
	if we get here then the line is not a comment and there
	are only two possibilities: a digit, which countes in
	the checksum and anything else (i.e., whitespace), which
	doesn't.
	add the digit into the w array. each word can hold 4 
	digits and the array is 16 words long. compute the
	sha every 16 words (64 digits). then continue reading
	stuff.
	this process will continue until we hit the end of
	the file.
	when we come out of this loop there is guaranteed to
	be at least one more empty word in the array, and iword
	is its index. note that this word may be partially
	full or it may be completely empty.
*/
	if(isdigit( (int) cc))	/*we got a live one*/
	   {
	   w[iword] <<= 8;	/*left shift the stuff already there*/
	   w[iword] |= cc;	/*put this character in the bottom*/
	   icount++;		/*byte counter*/
	   if(++ibyte == 4)	/* word is full*/
		{
		ibyte= 0;
		if(++iword == 16)	/*array is full*/
		   {
		   sha(w,h);
		   for(i=0; i<16; i++) w[i] = 0;	/*reset array*/
		   iword= 0;		/*reset the pointer*/
		}		/*end of array is full -- hash this stuff now*/
	   }			/*end of word is full*/
	}			/*end of character is a digit*/
	goto rdnxtchar;		/*get the next character*/
}
